/////////////////////////////////////////////////////////////////////////////
// Implementation for a MFC class encapsulation of the synchronous sockets

/*
 * Copyright (c) 2003 - 2005 by Daniel Weber (daniel@astolat.de)
 * All rights reserved.
 *
 * This is a SSL/TLS-implementation for Windows-Sockets in C++ written by
 * Daniel Weber (daniel@astolat.de) based an examples provided by
 * Martin Ziacek (martin.ziacek@blueyonder.co.uk).
 *
 * You are allowed to use, modify and redistribute this source-code for
 * commercial and non-commercial software as long as the following
 * conditions are met:
 * - Copyright remains bei Daniel Weber and redistributions of the source-
 *   code must retain this copyright-notice, list of conditions and
 *   disclaimer.
 * - Redistributions in binary form must mention Daniel Weber as author
 *   of parts of the SSL/TLS-implementations at program startup and/or
 *   in the documentation.
 *
 * This source-code is provided by Daniel Weber "as is" and any express or
 * implied warranties, including but not limited to the implied warranties
 * of merchantability and fitness for a particular purpose are disclaimed.
 * The authors or contributors are not liable for any damages arising out
 * of the use of this source-code, even if advised of the possibility of
 * such damage.
 */

#include "stdafx.h"

#include "SocketException.h"

/////////////////////////////////////////////////////////////////////////////
// Implementation for a MFC socket exception class

// Construction
CSocketException::CSocketException( int nError )
{
	m_nError = nError;
}

// Operations
CString CSocketException::GetErrorMessage()
{
	CString strError;
	switch( m_nError )
	{
	case WSAEINTR:
		strError = _T("the (blocking) call was canceled using WSACancelBlockingCall");
		break;
//	case WSAEBADF:
	case WSAEACCES:
		strError = _T("the requested address is a broadcast address, but the appropriate flag was not set");
		break;
	case WSAEFAULT:
		strError = _T("he buf argument is not in a valid part of the user address space or the nSockAddrLen argument is incorrect");
		break;
	case WSAEINVAL:
		strError = _T("the socket has not been bound to an address");
		break;
	case WSAEMFILE:
		strError = _T("no more file descriptors are available");
		break;

	case WSAEWOULDBLOCK:
		strError = _T("the socket is marked as nonblocking and the requested operation would block");
		break;
	case WSAEINPROGRESS:
		strError = _T("a blocking Windows Sockets call is in progress");
		break;
//	case WSAEALREADY:
	case WSAENOTSOCK:
		strError = _T("the descriptor is not a socket");
		break;
	case WSAEDESTADDRREQ:
		strError = _T("a destination address is required");
		break;
	case WSAEMSGSIZE:
		strError = _T("the datagram is larger than the maximum supported by the Windows Sockets implementation");
		break;
	case WSAEPROTOTYPE:
		strError = _T("the specified port is the wrong type for this socket");
		break;
//	case WSAENOPROTOOPT:
	case WSAEPROTONOSUPPORT:
		strError = _T("the specified port is not supported");
		break;
	case WSAESOCKTNOSUPPORT:
		strError = _T("the specified socket type is not supported in this address family");
		break;
	case WSAEOPNOTSUPP:
		strError = _T("MSG_OOB was specified, but the socket is not of type SOCK_STREAM");
		break;
//	case WSAEPFNOSUPPORT:
	case WSAEAFNOSUPPORT:
		strError = _T("addresses in the specified family cannot be used with this socket");
		break;
	case WSAEADDRINUSE:
		strError = _T("the specified address is already in use");
		break;
	case WSAEADDRNOTAVAIL:
		strError = _T("the specified address is not available from the local machine");
		break;
	case WSAENETDOWN:
		strError = _T("the Windows Sockets implementation detected that the network subsystem failed");
		break;
	case WSAENETUNREACH:
		strError = _T("the network cannot be reached from this host at this time");
		break;
	case WSAENETRESET:
		strError = _T("the connection must be reset because the Windows Sockets implementation dropped it");
		break;
	case WSAECONNABORTED:
		strError = _T("the connection was aborted due to timeout or other failure");
		break;
	case WSAECONNRESET:
		strError = _T("the connection was reset by the remote side");
		break;
	case WSAENOBUFS:
		strError = _T("the Windows Sockets implementation reports a buffer deadlock");
		break;
	case WSAEISCONN:
		strError = _T("the socket is already connected");
		break;
	case WSAENOTCONN:
		strError = _T("the socket is not connected");
		break;
	case WSAESHUTDOWN:
		strError = _T("the socket has been shut down");
		break;
//	case WSAETOOMANYREFS:
	case WSAETIMEDOUT:
		strError = _T("the attempt to connect timed out without establishing a connection");
		break;
	case WSAECONNREFUSED:
		strError = _T("the attempt to connect was rejected");
		break;
//	case WSAELOOP:
//	case WSAENAMETOOLONG:
//	case WSAEHOSTDOWN:
	case WSAEHOSTUNREACH:
		strError = _T("the host cannot be reached from this host at this time");
		break;
//	case WSAENOTEMPTY:
//	case WSAEPROCLIM:
//	case WSAEUSERS:
//	case WSAEDQUOT:
//	case WSAESTALE:
//	case WSAEREMOTE:

	case WSAEDISCON:
		strError = _T("the message terminated gracefully");
		break;

	case WSASYSNOTREADY:
		strError = _T("the network subsystem is unusable");
		break;
	case WSAVERNOTSUPPORTED:
		strError = _T("the Windows Sockets DLL does not support this application");
		break;
	case WSANOTINITIALISED:
		strError = _T("a successful AfxSocketInit must occur before using this API");
		break;

	case WSAHOST_NOT_FOUND:
		strError = _T("the host was not found");
		break;
//	case WSATRY_AGAIN:
//	case WSANO_RECOVERY:
//	case WSANO_DATA:
//	case WSANO_ADDRESS:

	default:
		strError.Format( _T("unknown error: %d"), m_nError );
		break;
	}

	return strError;
}
